@@ -155,6 +155,13 @@ impl Debug for Utf8Chunked {
     }
 }
 
+#[cfg(feature = "dtype-binary")]
+impl Debug for BinaryChunked {
+    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+        format_array!(f, self, "binary", self.name(), "ChunkedArray")
+    }
+}
+
 impl Debug for ListChunked {
     fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
         format_array!(f, self, "list", self.name(), "ChunkedArray")
@@ -276,6 +283,10 @@ impl Debug for Series {
             DataType::Null => {
                 writeln!(f, "nullarray")
             }
+            #[cfg(feature = "dtype-binary")]
+            DataType::Binary => {
+                format_array!(f, self.binary().unwrap(), "binary", self.name(), "Series")
+            }
             dt => panic!("{:?} not impl", dt),
         }
     }
@@ -510,6 +521,8 @@ impl Display for DataFrame {
 
             if std::env::var("POLARS_FMT_TABLE_HIDE_DATAFRAME_SHAPE_INFORMATION").is_ok() {
                 write!(f, "{}", table)?;
+            } else if std::env::var("POLARS_FMT_TABLE_DATAFRAME_SHAPE_BELOW").is_ok() {
+                write!(f, "{}\nshape: {:?}", table, self.shape())?;
             } else {
                 write!(f, "shape: {:?}\n{}", self.shape(), table)?;
             }
@@ -662,6 +675,8 @@ impl Display for AnyValue<'_> {
             AnyValue::Boolean(v) => write!(f, "{}", *v),
             AnyValue::Utf8(v) => write!(f, "{}", format_args!("\"{}\"", v)),
             AnyValue::Utf8Owned(v) => write!(f, "{}", format_args!("\"{}\"", v)),
+            #[cfg(feature = "dtype-binary")]
+            AnyValue::Binary(_) | AnyValue::BinaryOwned(_) => write!(f, "[binary data]"),
             #[cfg(feature = "dtype-date")]
             AnyValue::Date(v) => write!(f, "{}", date32_to_date(*v)),
             #[cfg(feature = "dtype-datetime")]
@@ -677,8 +692,9 @@ impl Display for AnyValue<'_> {
                         #[cfg(feature = "timezones")]
                         {
                             let tz = _tz.parse::<chrono_tz::Tz>().unwrap();
-                            let tz_dt = tz.from_local_datetime(&ndt).unwrap();
-                            write!(f, "{}", tz_dt)
+                            let dt_utc = chrono::Utc.from_local_datetime(&ndt).unwrap();
+                            let dt_tz_aware = dt_utc.with_timezone(&tz);
+                            write!(f, "{}", dt_tz_aware)
                         }
                         #[cfg(not(feature = "timezones"))]
                         {
@@ -775,6 +791,13 @@ impl FmtList for Utf8Chunked {
     }
 }
 
+#[cfg(feature = "dtype-binary")]
+impl FmtList for BinaryChunked {
+    fn fmt_list(&self) -> String {
+        impl_fmt_list!(self)
+    }
+}
+
 impl FmtList for ListChunked {
     fn fmt_list(&self) -> String {
         impl_fmt_list!(self)
