@@ -114,6 +114,11 @@ impl Schema {
         self.inner.get(name)
     }
 
+    pub fn try_get(&self, name: &str) -> PolarsResult<&DataType> {
+        self.get(name)
+            .ok_or_else(|| PolarsError::NotFound(name.to_string().into()))
+    }
+
     pub fn get_full(&self, name: &str) -> Option<(usize, &String, &DataType)> {
         self.inner.get_full(name)
     }
@@ -124,6 +129,13 @@ impl Schema {
             .map(|dtype| Field::new(name, dtype.clone()))
     }
 
+    pub fn try_get_field(&self, name: &str) -> PolarsResult<Field> {
+        self.inner
+            .get(name)
+            .ok_or_else(|| PolarsError::NotFound(name.to_string().into()))
+            .map(|dtype| Field::new(name, dtype.clone()))
+    }
+
     pub fn get_index(&self, index: usize) -> Option<(&String, &DataType)> {
         self.inner.get_index(index)
     }
