@@ -2,7 +2,6 @@ use std::borrow::Cow;
 use std::ops::{BitAnd, BitOr, BitXor};
 
 use ahash::RandomState;
-use polars_arrow::prelude::QuantileInterpolOptions;
 
 use super::{private, IntoSeries, SeriesTrait, *};
 use crate::chunked_array::comparison::*;
@@ -51,8 +50,9 @@ impl private::PrivateSeries for SeriesWrap<BooleanChunked> {
         (&self.0).into_partial_ord_inner()
     }
 
-    fn vec_hash(&self, random_state: RandomState) -> PolarsResult<Vec<u64>> {
-        Ok(self.0.vec_hash(random_state))
+    fn vec_hash(&self, random_state: RandomState, buf: &mut Vec<u64>) -> PolarsResult<()> {
+        self.0.vec_hash(random_state, buf);
+        Ok(())
     }
 
     fn vec_hash_combine(&self, build_hasher: RandomState, hashes: &mut [u64]) -> PolarsResult<()> {
@@ -332,23 +332,6 @@ impl SeriesTrait for SeriesWrap<BooleanChunked> {
     fn min_as_series(&self) -> Series {
         ChunkAggSeries::min_as_series(&self.0)
     }
-    fn median_as_series(&self) -> Series {
-        QuantileAggSeries::median_as_series(&self.0)
-    }
-    fn var_as_series(&self, ddof: u8) -> Series {
-        VarAggSeries::var_as_series(&self.0, ddof)
-    }
-    fn std_as_series(&self, ddof: u8) -> Series {
-        VarAggSeries::std_as_series(&self.0, ddof)
-    }
-    fn quantile_as_series(
-        &self,
-        quantile: f64,
-        interpol: QuantileInterpolOptions,
-    ) -> PolarsResult<Series> {
-        QuantileAggSeries::quantile_as_series(&self.0, quantile, interpol)
-    }
-
     fn fmt_list(&self) -> String {
         FmtList::fmt_list(&self.0)
     }
