@@ -117,11 +117,15 @@ pub(crate) mod private {
         fn into_partial_ord_inner<'a>(&'a self) -> Box<dyn PartialOrdInner + 'a> {
             invalid_operation_panic!(self)
         }
-        fn vec_hash(&self, _build_hasher: RandomState) -> Vec<u64> {
-            invalid_operation_panic!(self)
+        fn vec_hash(&self, _build_hasher: RandomState, _buf: &mut Vec<u64>) -> PolarsResult<()> {
+            invalid_operation!(self)
         }
-        fn vec_hash_combine(&self, _build_hasher: RandomState, _hashes: &mut [u64]) {
-            invalid_operation_panic!(self)
+        fn vec_hash_combine(
+            &self,
+            _build_hasher: RandomState,
+            _hashes: &mut [u64],
+        ) -> PolarsResult<()> {
+            invalid_operation!(self)
         }
         unsafe fn agg_min(&self, groups: &GroupsProxy) -> Series {
             Series::full_null(self._field().name(), groups.len(), self._dtype())
@@ -143,17 +147,7 @@ pub(crate) mod private {
         unsafe fn agg_list(&self, groups: &GroupsProxy) -> Series {
             Series::full_null(self._field().name(), groups.len(), self._dtype())
         }
-        unsafe fn agg_quantile(
-            &self,
-            groups: &GroupsProxy,
-            _quantile: f64,
-            _interpol: QuantileInterpolOptions,
-        ) -> Series {
-            Series::full_null(self._field().name(), groups.len(), self._dtype())
-        }
-        unsafe fn agg_median(&self, groups: &GroupsProxy) -> Series {
-            Series::full_null(self._field().name(), groups.len(), self._dtype())
-        }
+
         fn zip_outer_join_column(
             &self,
             _right_column: &Series,
@@ -177,8 +171,8 @@ pub(crate) mod private {
         fn remainder(&self, _rhs: &Series) -> PolarsResult<Series> {
             invalid_operation_panic!(self)
         }
-        fn group_tuples(&self, _multithreaded: bool, _sorted: bool) -> GroupsProxy {
-            invalid_operation_panic!(self)
+        fn group_tuples(&self, _multithreaded: bool, _sorted: bool) -> PolarsResult<GroupsProxy> {
+            invalid_operation!(self)
         }
         fn zip_with_same_type(
             &self,
@@ -368,7 +362,7 @@ pub trait SeriesTrait:
 
     /// Drop all null values and return a new Series.
     fn drop_nulls(&self) -> Series {
-        if !self.has_validity() {
+        if self.null_count() == 0 {
             Series(self.clone_inner())
         } else {
             self.filter(&self.is_not_null()).unwrap()
@@ -587,15 +581,15 @@ pub trait SeriesTrait:
     }
     /// Get the median of the Series as a new Series of length 1.
     fn median_as_series(&self) -> Series {
-        invalid_operation_panic!(self)
+        Series::full_null(self.name(), 1, self.dtype())
     }
     /// Get the variance of the Series as a new Series of length 1.
     fn var_as_series(&self, _ddof: u8) -> Series {
-        invalid_operation_panic!(self)
+        Series::full_null(self.name(), 1, self.dtype())
     }
     /// Get the standard deviation of the Series as a new Series of length 1.
     fn std_as_series(&self, _ddof: u8) -> Series {
-        invalid_operation_panic!(self)
+        Series::full_null(self.name(), 1, self.dtype())
     }
     /// Get the quantile of the ChunkedArray as a new Series of length 1.
     fn quantile_as_series(
@@ -603,7 +597,7 @@ pub trait SeriesTrait:
         _quantile: f64,
         _interpol: QuantileInterpolOptions,
     ) -> PolarsResult<Series> {
-        invalid_operation_panic!(self)
+        Ok(Series::full_null(self.name(), 1, self.dtype()))
     }
 
     fn fmt_list(&self) -> String {
