@@ -21,7 +21,7 @@ pub trait VecHash {
     /// Compute the hash for all values in the array.
     ///
     /// This currently only works with the AHash RandomState hasher builder.
-    fn vec_hash(&self, _random_state: RandomState) -> Vec<u64> {
+    fn vec_hash(&self, _random_state: RandomState, _buf: &mut Vec<u64>) {
         unimplemented!()
     }
 
@@ -46,17 +46,18 @@ where
     T: PolarsIntegerType,
     T::Native: Hash + CallHasher,
 {
-    fn vec_hash(&self, random_state: RandomState) -> Vec<u64> {
+    fn vec_hash(&self, random_state: RandomState, buf: &mut Vec<u64>) {
         // Note that we don't use the no null branch! This can break in unexpected ways.
         // for instance with threading we split an array in n_threads, this may lead to
         // splits that have no nulls and splits that have nulls. Then one array is hashed with
         // Option<T> and the other array with T.
         // Meaning that they cannot be compared. By always hashing on Option<T> the random_state is
         // the only deterministic seed.
-        let mut av = Vec::with_capacity(self.len());
+        buf.clear();
+        buf.reserve(self.len());
 
         self.downcast_iter().for_each(|arr| {
-            av.extend(
+            buf.extend(
                 arr.values()
                     .as_slice()
                     .iter()
@@ -65,7 +66,7 @@ where
         });
 
         let null_h = get_null_hash_value(random_state);
-        let hashes = av.as_mut_slice();
+        let hashes = buf.as_mut_slice();
 
         let mut offset = 0;
         self.downcast_iter().for_each(|arr| {
@@ -80,8 +81,6 @@ where
             }
             offset += arr.len();
         });
-
-        av
     }
 
     fn vec_hash_combine(&self, random_state: RandomState, hashes: &mut [u64]) {
@@ -120,16 +119,16 @@ where
 }
 
 impl VecHash for Utf8Chunked {
-    fn vec_hash(&self, random_state: RandomState) -> Vec<u64> {
+    fn vec_hash(&self, random_state: RandomState, buf: &mut Vec<u64>) {
+        buf.clear();
+        buf.reserve(self.len());
         let null_h = get_null_hash_value(random_state.clone());
-        let mut av = Vec::with_capacity(self.len());
         self.downcast_iter().for_each(|arr| {
-            av.extend(arr.into_iter().map(|opt_v| match opt_v {
+            buf.extend(arr.into_iter().map(|opt_v| match opt_v {
                 Some(v) => str::get_hash(v, &random_state),
                 None => null_h,
             }))
         });
-        av
     }
 
     fn vec_hash_combine(&self, random_state: RandomState, hashes: &mut [u64]) {
@@ -147,17 +146,46 @@ impl VecHash for Utf8Chunked {
     }
 }
 
-impl VecHash for BooleanChunked {
-    fn vec_hash(&self, random_state: RandomState) -> Vec<u64> {
-        let mut av = Vec::with_capacity(self.len());
+#[cfg(feature = "dtype-binary")]
+impl VecHash for BinaryChunked {
+    fn vec_hash(&self, random_state: RandomState, buf: &mut Vec<u64>) {
+        buf.clear();
+        buf.reserve(self.len());
+        let null_h = get_null_hash_value(random_state.clone());
         self.downcast_iter().for_each(|arr| {
-            av.extend(arr.into_iter().map(|opt_v| {
+            buf.extend(arr.into_iter().map(|opt_v| match opt_v {
+                Some(v) => <[u8]>::get_hash(v, &random_state),
+                None => null_h,
+            }))
+        });
+    }
+
+    fn vec_hash_combine(&self, random_state: RandomState, hashes: &mut [u64]) {
+        let null_h = get_null_hash_value(random_state.clone());
+        self.apply_to_slice(
+            |opt_v, h| {
+                let l = match opt_v {
+                    Some(v) => <[u8]>::get_hash(v, &random_state),
+                    None => null_h,
+                };
+                _boost_hash_combine(l, *h)
+            },
+            hashes,
+        )
+    }
+}
+
+impl VecHash for BooleanChunked {
+    fn vec_hash(&self, random_state: RandomState, buf: &mut Vec<u64>) {
+        buf.clear();
+        buf.reserve(self.len());
+        self.downcast_iter().for_each(|arr| {
+            buf.extend(arr.into_iter().map(|opt_v| {
                 let mut hasher = random_state.build_hasher();
                 opt_v.hash(&mut hasher);
                 hasher.finish()
             }))
         });
-        av
     }
 
     fn vec_hash_combine(&self, random_state: RandomState, hashes: &mut [u64]) {
@@ -173,8 +201,8 @@ impl VecHash for BooleanChunked {
 }
 
 impl VecHash for Float32Chunked {
-    fn vec_hash(&self, random_state: RandomState) -> Vec<u64> {
-        self.bit_repr_small().vec_hash(random_state)
+    fn vec_hash(&self, random_state: RandomState, buf: &mut Vec<u64>) {
+        self.bit_repr_small().vec_hash(random_state, buf)
     }
 
     fn vec_hash_combine(&self, random_state: RandomState, hashes: &mut [u64]) {
@@ -182,8 +210,8 @@ impl VecHash for Float32Chunked {
     }
 }
 impl VecHash for Float64Chunked {
-    fn vec_hash(&self, random_state: RandomState) -> Vec<u64> {
-        self.bit_repr_large().vec_hash(random_state)
+    fn vec_hash(&self, random_state: RandomState, buf: &mut Vec<u64>) {
+        self.bit_repr_large().vec_hash(random_state, buf)
     }
     fn vec_hash_combine(&self, random_state: RandomState, hashes: &mut [u64]) {
         self.bit_repr_large().vec_hash_combine(random_state, hashes)
@@ -195,23 +223,23 @@ impl<T> VecHash for ObjectChunked<T>
 where
     T: PolarsObject,
 {
-    fn vec_hash(&self, random_state: RandomState) -> Vec<u64> {
+    fn vec_hash(&self, random_state: RandomState, buf: &mut Vec<u64>) {
         // Note that we don't use the no null branch! This can break in unexpected ways.
         // for instance with threading we split an array in n_threads, this may lead to
         // splits that have no nulls and splits that have nulls. Then one array is hashed with
         // Option<T> and the other array with T.
         // Meaning that they cannot be compared. By always hashing on Option<T> the random_state is
         // the only deterministic seed.
-        let mut av = Vec::with_capacity(self.len());
+        buf.clear();
+        buf.reserve(self.len());
 
         self.downcast_iter().for_each(|arr| {
-            av.extend(arr.into_iter().map(|opt_v| {
+            buf.extend(arr.into_iter().map(|opt_v| {
                 let mut hasher = random_state.build_hasher();
                 opt_v.hash(&mut hasher);
                 hasher.finish()
             }))
         });
-        av
     }
 
     fn vec_hash_combine(&self, random_state: RandomState, hashes: &mut [u64]) {
@@ -574,7 +602,8 @@ pub(crate) fn df_rows_to_hashes(
 
     let mut iter = keys.iter();
     let first = iter.next().expect("at least one key");
-    let mut hashes = first.vec_hash(build_hasher.clone())?;
+    let mut hashes = vec![];
+    first.vec_hash(build_hasher.clone(), &mut hashes)?;
     let hslice = hashes.as_mut_slice();
 
     for keys in iter {
