@@ -1,10 +1,5 @@
 #![cfg_attr(docsrs, feature(doc_cfg))]
 
-#[cfg(feature = "private")]
-pub mod aggregations;
-#[cfg(not(feature = "private"))]
-pub(crate) mod aggregations;
-
 #[cfg(feature = "avro")]
 #[cfg_attr(docsrs, doc(cfg(feature = "avro")))]
 pub mod avro;
@@ -57,13 +52,6 @@ pub use options::*;
 use polars_core::frame::ArrowChunk;
 use polars_core::prelude::*;
 
-#[cfg(any(
-    feature = "ipc",
-    feature = "json",
-    feature = "avro",
-    feature = "ipc_streaming"
-))]
-use crate::aggregations::{apply_aggregations, ScanAggregation};
 #[cfg(any(
     feature = "ipc",
     feature = "json",
@@ -88,7 +76,7 @@ where
     }
 
     /// Take the SerReader and return a parsed DataFrame.
-    fn finish(self) -> Result<DataFrame>;
+    fn finish(self) -> PolarsResult<DataFrame>;
 }
 
 pub trait SerWriter<W>
@@ -98,7 +86,7 @@ where
     fn new(writer: W) -> Self
     where
         Self: Sized;
-    fn finish(&mut self, df: &mut DataFrame) -> Result<()>;
+    fn finish(&mut self, df: &mut DataFrame) -> PolarsResult<()>;
 }
 
 pub trait WriterFactory {
@@ -121,10 +109,9 @@ pub(crate) fn finish_reader<R: ArrowReader>(
     rechunk: bool,
     n_rows: Option<usize>,
     predicate: Option<Arc<dyn PhysicalIoExpr>>,
-    aggregate: Option<&[ScanAggregation]>,
     arrow_schema: &ArrowSchema,
     row_count: Option<RowCount>,
-) -> Result<DataFrame> {
+) -> PolarsResult<DataFrame> {
     use polars_core::utils::accumulate_dataframes_vertical;
 
     let mut num_rows = 0;
@@ -145,8 +132,6 @@ pub(crate) fn finish_reader<R: ArrowReader>(
             df = df.filter(mask)?;
         }
 
-        apply_aggregations(&mut df, aggregate)?;
-
         if let Some(n) = n_rows {
             if num_rows >= n {
                 let len = n - parsed_dfs
@@ -172,14 +157,11 @@ pub(crate) fn finish_reader<R: ArrowReader>(
                 .map(|fld| {
                     Series::try_from((fld.name.as_str(), new_empty_array(fld.data_type.clone())))
                 })
-                .collect::<Result<_>>()?;
+                .collect::<PolarsResult<_>>()?;
             DataFrame::new(empty_cols)?
         } else {
             // If there are any rows, accumulate them into a df
-            let mut df = accumulate_dataframes_vertical(parsed_dfs)?;
-            // Aggregations must be applied a final time to aggregate the partitions
-            apply_aggregations(&mut df, aggregate)?;
-            df
+            accumulate_dataframes_vertical(parsed_dfs)?
         }
     };
 
