@@ -228,6 +228,23 @@ fn test_escape_double_quotes() {
     )));
 }
 
+#[test]
+fn test_newline_in_custom_quote_char() {
+    // newline inside custom quote char (default is ") should parse correctly
+    let csv = r#"column_1,column_2
+        1,'foo
+        bar'
+        2,'bar'
+"#;
+
+    let file = Cursor::new(csv);
+    let df = CsvReader::new(file)
+        .with_quote_char(Some(b'\''))
+        .finish()
+        .unwrap();
+    assert_eq!(df.shape(), (2, 2));
+}
+
 #[test]
 fn test_escape_2() {
     // this is is harder than it looks.
