@@ -21,6 +21,7 @@ from polars.convert import (
     from_records,
 )
 from polars.datatypes import (
+    Binary,
     Boolean,
     Categorical,
     DataType,
@@ -59,6 +60,7 @@ from polars.exceptions import (
     SchemaError,
     ShapeError,
 )
+from polars.internals import BatchedCsvReader
 
 # TODO remove need for wrap_df
 from polars.internals.dataframe import wrap_df  # noqa: F401
@@ -89,6 +91,8 @@ from polars.internals.lazy_functions import (
     concat_str,
     count,
     cov,
+    cumfold,
+    cumsum,
     duration,
     element,
     exclude,
@@ -127,6 +131,7 @@ from polars.internals.whenthen import when
 from polars.io import (
     read_avro,
     read_csv,
+    read_csv_batched,
     read_excel,
     read_ipc,
     read_json,
@@ -149,6 +154,7 @@ __all__ = [
     "ShapeError",
     "SchemaError",
     "ArrowError",
+    "BatchedCsvReader",
     "ComputeError",
     "InvalidOperationError",
     "NoDataError",
@@ -170,6 +176,7 @@ __all__ = [
     "Float32",
     "Float64",
     "Boolean",
+    "Binary",
     "Utf8",
     "List",
     "Date",
@@ -186,6 +193,7 @@ __all__ = [
     "get_idx_type",
     # polars.io
     "read_csv",
+    "read_csv_batched",
     "read_excel",
     "read_parquet",
     "read_json",
@@ -242,6 +250,8 @@ __all__ = [
     "map",
     "apply",
     "fold",
+    "cumfold",
+    "cumsum",
     "any",
     "all",
     "groups",
