@@ -269,19 +269,40 @@ def test_set_tbl_rows(environ: None) -> None:
     )
 
 
+def test_shape_below_table(environ: None) -> None:
+    df = pl.DataFrame({"a": [1, 2], "b": [3, 4], "c": [5, 6]})
+    pl.Config.set_tbl_dataframe_shape_below(True)
+    assert (
+        str(df) == "┌─────┬─────┬─────┐\n"
+        "│ a   ┆ b   ┆ c   │\n"
+        "│ --- ┆ --- ┆ --- │\n"
+        "│ i64 ┆ i64 ┆ i64 │\n"
+        "╞═════╪═════╪═════╡\n"
+        "│ 1   ┆ 3   ┆ 5   │\n"
+        "├╌╌╌╌╌┼╌╌╌╌╌┼╌╌╌╌╌┤\n"
+        "│ 2   ┆ 4   ┆ 6   │\n"
+        "└─────┴─────┴─────┘\n"
+        "shape: (2, 3)"
+    )
+
+
 def test_string_cache(environ: None) -> None:
     df1 = pl.DataFrame({"a": ["foo", "bar", "ham"], "b": [1, 2, 3]})
     df2 = pl.DataFrame({"a": ["foo", "spam", "eggs"], "c": [3, 2, 2]})
 
     # ensure cache is off when casting to categorical; the join will fail
-    pl.Config.unset_global_string_cache()
+    pl.toggle_string_cache(False)
+    assert pl.using_string_cache() is False
+
     df1a = df1.with_column(pl.col("a").cast(pl.Categorical))
     df2a = df2.with_column(pl.col("a").cast(pl.Categorical))
     with pytest.raises(pl.ComputeError):
         _ = df1a.join(df2a, on="a", how="inner")
 
     # now turn on the cache
-    pl.Config.set_global_string_cache()
+    pl.toggle_string_cache(True)
+    assert pl.using_string_cache() is True
+
     df1b = df1.with_column(pl.col("a").cast(pl.Categorical))
     df2b = df2.with_column(pl.col("a").cast(pl.Categorical))
     out = df1b.join(df2b, on="a", how="inner")
@@ -289,4 +310,59 @@ def test_string_cache(environ: None) -> None:
 
     # turn off again so we do not break other tests
     # (TODO: environ fixture does not roll this back?)
-    pl.Config.unset_global_string_cache()
+    pl.toggle_string_cache(False)
+    assert pl.using_string_cache() is False
+
+
+def test_config_load_save(environ: None) -> None:
+    # set some config options
+    pl.Config.with_columns_kwargs = True
+    pl.Config.set_verbose(True)
+    assert os.environ["POLARS_VERBOSE"] == "1"
+
+    cfg = pl.Config.save()
+    assert isinstance(cfg, str)
+    assert "POLARS_VERBOSE" in pl.Config.state(if_set=True)
+
+    # unset the saved options
+    pl.Config.with_columns_kwargs = False
+    pl.Config.set_verbose(False)
+    assert os.environ["POLARS_VERBOSE"] == "0"
+
+    # now load back from config...
+    pl.Config.load(cfg)
+
+    # ...and confirm the saved options were set
+    assert os.environ["POLARS_VERBOSE"] == "1"
+    assert pl.Config.with_columns_kwargs is True
+
+    # restore explicitly-set config options (unsets from env)
+    pl.Config.restore_defaults()
+    assert "POLARS_VERBOSE" not in pl.Config.state(if_set=True)
+    assert "POLARS_VERBOSE" in pl.Config.state()
+    assert os.environ.get("POLARS_VERBOSE") is None
+    assert pl.Config.with_columns_kwargs is False
+
+
+def test_config_context(environ: None) -> None:
+    pl.Config.set_verbose(False)
+    pl.Config.set_tbl_cols(8)
+
+    initial_state = pl.Config.state()
+
+    with pl.Config() as cfg:
+        cfg.set_verbose(True).set_tbl_hide_column_separator(True).set_ascii_tables()
+
+        new_state_entries = set(
+            {
+                "POLARS_FMT_MAX_COLS": "8",
+                "POLARS_FMT_TABLE_FORMATTING": "ASCII_FULL",
+                "POLARS_FMT_TABLE_HIDE_COLUMN_SEPARATOR": "1",
+                "POLARS_VERBOSE": "1",
+            }.items()
+        )
+        assert set(initial_state.items()) != new_state_entries
+        assert new_state_entries.issubset(set(cfg.state().items()))
+
+    # expect scope-exit to restore original state
+    assert pl.Config.state() == initial_state
