@@ -17,7 +17,11 @@ else:
 
 import polars as pl
 from polars.datatypes import DTYPE_TEMPORAL_UNITS, TemporalDataType
-from polars.testing import assert_series_equal, verify_series_and_expr_api
+from polars.testing import (
+    assert_frame_equal,
+    assert_series_equal,
+    verify_series_and_expr_api,
+)
 
 if TYPE_CHECKING:
     from polars.internals.type_aliases import TimeUnit
@@ -49,23 +53,37 @@ def test_fill_null() -> None:
 
 
 def test_filter_date() -> None:
-    dataset = pl.DataFrame(
+    dtcol = pl.col("date")
+    df = pl.DataFrame(
         {"date": ["2020-01-02", "2020-01-03", "2020-01-04"], "index": [1, 2, 3]}
-    )
-    df = dataset.with_column(pl.col("date").str.strptime(pl.Date, "%Y-%m-%d"))
-    assert df.filter(pl.col("date") <= pl.lit(datetime(2019, 1, 3))).is_empty()
-    assert df.filter(pl.col("date") < pl.lit(datetime(2020, 1, 4))).shape[0] == 2
-    assert df.filter(pl.col("date") < pl.lit(datetime(2020, 1, 5))).shape[0] == 3
-    assert df.filter(pl.col("date") <= pl.lit(date(2019, 1, 3))).is_empty()
-    assert df.filter(pl.col("date") < pl.lit(date(2020, 1, 4))).shape[0] == 2
-    assert df.filter(pl.col("date") < pl.lit(date(2020, 1, 5))).shape[0] == 3
+    ).with_column(dtcol.str.strptime(pl.Date, "%Y-%m-%d"))
+    assert df.rows() == [
+        (date(2020, 1, 2), 1),
+        (date(2020, 1, 3), 2),
+        (date(2020, 1, 4), 3),
+    ]
+
+    # filter by datetime
+    assert df.filter(dtcol <= pl.lit(datetime(2019, 1, 3))).is_empty()
+    assert df.filter(dtcol < pl.lit(datetime(2020, 1, 4))).rows() == df.rows()[:2]
+    assert df.filter(dtcol < pl.lit(datetime(2020, 1, 5))).rows() == df.rows()
+
+    # filter by date
+    assert df.filter(dtcol <= pl.lit(date(2019, 1, 3))).is_empty()
+    assert df.filter(dtcol < pl.lit(date(2020, 1, 4))).rows() == df.rows()[:2]
+    assert df.filter(dtcol < pl.lit(date(2020, 1, 5))).rows() == df.rows()
 
 
 def test_filter_time() -> None:
-    df = pl.DataFrame({"t": [time(8, 0), time(9, 0), time(10, 0)]})
+    times = [time(8, 0), time(9, 0), time(10, 0)]
+    df = pl.DataFrame({"t": times})
+
     assert df.filter(pl.col("t") <= pl.lit(time(7, 0))).is_empty()
-    assert df.filter(pl.col("t") < pl.lit(time(10, 0))).shape[0] == 2
-    assert df.filter(pl.col("t") < pl.lit(time(11, 0))).shape[0] == 3
+    assert df.filter(pl.col("t") < pl.lit(time(11, 0))).rows() == [(t,) for t in times]
+    assert df.filter(pl.col("t") < pl.lit(time(10, 0))).to_series().to_list() == [
+        time(8, 0),
+        time(9, 0),
+    ]
 
 
 def test_series_add_timedelta() -> None:
@@ -94,7 +112,6 @@ def test_diff_datetime() -> None:
             "char": ["a", "a", "b"],
         }
     )
-
     out = (
         df.with_columns(
             [
@@ -457,7 +474,7 @@ def test_truncate_negative_offset() -> None:
     )
 
     assert out["event_date"].to_list() == [
-        datetime(2021, 4, 1),
+        datetime(2021, 3, 1),
         datetime(2021, 4, 1),
         datetime(2021, 5, 1),
     ]
@@ -481,6 +498,7 @@ def test_truncate_negative_offset() -> None:
         every="1mo",
         by=["admin", "five_type", "actor"],
     ).agg([pl.col("adm1_code").unique(), (pl.col("fatalities") > 0).sum()])
+
     assert out["event_date"].to_list() == [
         datetime(2021, 4, 1),
         datetime(2021, 5, 1),
@@ -498,7 +516,9 @@ def test_truncate_negative_offset() -> None:
         out = df.groupby_dynamic(
             "idx", every="2i", period="3i", include_boundaries=True
         ).agg(pl.col("A").list())
+
         assert out.shape == (3, 4)
+        assert out["A"].to_list() == [["A", "A", "B"], ["B", "B", "B"], ["B", "C"]]
 
 
 def test_to_arrow() -> None:
@@ -543,20 +563,26 @@ def test_explode_date() -> None:
         date(2021, 12, 1),
         date(2021, 12, 1),
     ]
-    for d in [dates, datetimes]:
+    for dclass, values in ((date, dates), (datetime, datetimes)):
         df = pl.DataFrame(
             {
-                "a": d,
+                "a": values,
                 "b": ["a", "b", "a", "b"],
-                "c": [1.0, 2.0, 1.1, 2.2],
+                "c": [1.0, 2.0, 1.5, 2.5],
             }
         )
         out = (
-            df.groupby("b")
+            df.groupby("b", maintain_order=True)
             .agg([pl.col("a"), pl.col("c").pct_change()])
             .explode(["a", "c"])
         )
         assert out.shape == (4, 3)
+        assert out.rows() == [
+            ("a", dclass(2021, 12, 1), None),
+            ("a", dclass(2021, 12, 1), 0.5),
+            ("b", dclass(2021, 12, 1), None),
+            ("b", dclass(2021, 12, 1), 0.25),
+        ]
 
 
 def test_rolling() -> None:
@@ -572,7 +598,6 @@ def test_rolling() -> None:
     df = pl.DataFrame({"dt": dates, "a": [3, 7, 5, 9, 2, 1]}).with_column(
         pl.col("dt").str.strptime(pl.Datetime)
     )
-
     out = df.groupby_rolling(index_column="dt", period="2d").agg(
         [
             pl.sum("a").alias("sum_a"),
@@ -580,7 +605,6 @@ def test_rolling() -> None:
             pl.max("a").alias("max_a"),
         ]
     )
-
     assert out["sum_a"].to_list() == [3, 10, 15, 24, 11, 1]
     assert out["max_a"].to_list() == [3, 7, 7, 9, 9, 1]
     assert out["min_a"].to_list() == [3, 3, 3, 3, 2, 1]
@@ -677,7 +701,8 @@ def test_read_utc_times_parquet() -> None:
     df.to_parquet(f)
     f.seek(0)
     df_in = pl.read_parquet(f)
-    assert df_in["Timestamp"][0] == datetime(2022, 1, 1, 0, 0)
+    tz = zoneinfo.ZoneInfo("UTC")
+    assert df_in["Timestamp"][0] == datetime(2022, 1, 1, 0, 0, tzinfo=tz)
 
 
 def test_epoch() -> None:
@@ -708,9 +733,9 @@ def test_default_negative_every_offset_dynamic_groupby() -> None:
     expected = pl.DataFrame(
         {
             "dt": [
+                datetime(2019, 12, 1, 0, 0),
                 datetime(2020, 1, 1, 0, 0),
-                datetime(2020, 1, 1, 0, 0),
-                datetime(2020, 3, 1, 0, 0),
+                datetime(2020, 2, 1, 0, 0),
             ],
             "idx": [[0], [1, 2], [3]],
         }
@@ -979,15 +1004,46 @@ def test_asof_join() -> None:
     ].to_list() == [51.95, 51.95, None, None, 720.77, None, None, None]
 
 
-def test_lambda_with_python_datetime_return_type() -> None:
-    df = pl.DataFrame({"timestamp": [1284286794, 1234567890]})
+def test_temporal_dtypes_apply() -> None:
+    df = pl.DataFrame(
+        {"timestamp": [1284286794000, None, 1234567890000]},
+        columns=[("timestamp", pl.Datetime("ms"))],
+    )
+    const_dtm = datetime(2010, 9, 12)
 
-    assert df.with_column(
-        pl.col("timestamp").apply(lambda x: datetime(2010, 9, 12)).alias("my_date_time")
-    )["my_date_time"].to_list() == [
-        datetime(2010, 9, 12),
-        datetime(2010, 9, 12),
-    ]
+    assert_frame_equal(
+        df.with_columns(
+            [
+                # don't actually do any of this; native expressions are MUCH faster ;)
+                pl.col("timestamp").apply(lambda x: const_dtm).alias("const_dtm"),
+                pl.col("timestamp").apply(lambda x: x and x.date()).alias("date"),
+                pl.col("timestamp").apply(lambda x: x and x.time()).alias("time"),
+            ]
+        ),
+        pl.DataFrame(
+            [
+                (
+                    datetime(2010, 9, 12, 10, 19, 54),
+                    datetime(2010, 9, 12, 0, 0),
+                    date(2010, 9, 12),
+                    time(10, 19, 54),
+                ),
+                (None, const_dtm, None, None),
+                (
+                    datetime(2009, 2, 13, 23, 31, 30),
+                    datetime(2010, 9, 12, 0, 0),
+                    date(2009, 2, 13),
+                    time(23, 31, 30),
+                ),
+            ],
+            columns={
+                "timestamp": pl.Datetime("ms"),
+                "const_dtm": pl.Datetime("us"),
+                "date": pl.Date,
+                "time": pl.Time,
+            },
+        ),
+    )
 
 
 def test_timelike_init() -> None:
@@ -1001,15 +1057,20 @@ def test_timelike_init() -> None:
 
 
 def test_duration_filter() -> None:
-    date_df = pl.DataFrame(
+    df = pl.DataFrame(
         {
             "start_date": [date(2022, 1, 1), date(2022, 1, 1), date(2022, 1, 1)],
             "end_date": [date(2022, 1, 7), date(2022, 2, 20), date(2023, 1, 1)],
         }
     ).with_column((pl.col("end_date") - pl.col("start_date")).alias("time_passed"))
 
-    assert date_df.filter(pl.col("time_passed") < timedelta(days=30)).shape[0] == 1
-    assert date_df.filter(pl.col("time_passed") >= timedelta(days=30)).shape[0] == 2
+    assert df.filter(pl.col("time_passed") < timedelta(days=30)).rows() == [
+        (date(2022, 1, 1), date(2022, 1, 7), timedelta(days=6))
+    ]
+    assert df.filter(pl.col("time_passed") >= timedelta(days=30)).rows() == [
+        (date(2022, 1, 1), date(2022, 2, 20), timedelta(days=50)),
+        (date(2022, 1, 1), date(2023, 1, 1), timedelta(days=365)),
+    ]
 
 
 def test_agg_logical() -> None:
@@ -1021,14 +1082,18 @@ def test_agg_logical() -> None:
 
 @no_type_check
 def test_from_time_arrow() -> None:
-    times = pa.array([10, 20, 30], type=pa.time32("s"))
-    times_table = pa.table([times], names=["times"])
+    pa_times = pa.table([pa.array([10, 20, 30], type=pa.time32("s"))], names=["times"])
 
-    assert pl.from_arrow(times_table).to_series().to_list() == [
+    assert pl.from_arrow(pa_times).to_series().to_list() == [
         time(0, 0, 10),
         time(0, 0, 20),
         time(0, 0, 30),
     ]
+    assert pl.from_arrow(pa_times).rows() == [
+        (time(0, 0, 10),),
+        (time(0, 0, 20),),
+        (time(0, 0, 30),),
+    ]
 
 
 def test_datetime_strptime_patterns() -> None:
@@ -1371,6 +1436,22 @@ def test_time_microseconds_3843() -> None:
     assert s.to_list() == in_val
 
 
+def test_date_to_time_cast_5111() -> None:
+    # check date -> time casts (fast-path: always 00:00:00)
+    df = pl.DataFrame(
+        {
+            "xyz": [
+                date(1969, 1, 1),
+                date(1990, 3, 8),
+                date(2000, 6, 16),
+                date(2010, 9, 24),
+                date(2022, 12, 31),
+            ]
+        }
+    ).with_column(pl.col("xyz").cast(pl.Time))
+    assert df["xyz"].to_list() == [time(0), time(0), time(0), time(0), time(0)]
+
+
 def test_year_empty_df() -> None:
     df = pl.DataFrame(pl.Series(name="date", dtype=pl.Date))
     assert df.select(pl.col("date").dt.year()).dtypes == [pl.Int32]
@@ -1420,6 +1501,7 @@ def test_weekday() -> None:
     assert s.cast(pl.Date).dt.weekday()[0] == 0
 
 
+@pytest.mark.skip(reason="from_dicts cannot yet infer timezones")
 def test_from_dict_tu_consistency() -> None:
     tz = zoneinfo.ZoneInfo("PRC")
     dt = datetime(2020, 8, 1, 12, 0, 0, tzinfo=tz)
@@ -1527,6 +1609,8 @@ def test_invalid_date_parsing_4898() -> None:
 
 
 def test_cast_timezone() -> None:
+    utc = zoneinfo.ZoneInfo("UTC")
+    ny = zoneinfo.ZoneInfo("America/New_York")
     assert pl.DataFrame({"a": [datetime(2022, 9, 25, 14)]}).with_column(
         pl.col("a")
         .dt.with_time_zone("America/New_York")
@@ -1534,7 +1618,7 @@ def test_cast_timezone() -> None:
         .alias("b")
     ).to_dict(False) == {
         "a": [datetime(2022, 9, 25, 14, 0)],
-        "b": [datetime(2022, 9, 25, 18, 0)],
+        "b": [datetime(2022, 9, 25, 18, 0, tzinfo=utc)],
     }
     assert pl.DataFrame({"a": [datetime(2022, 9, 25, 18)]}).with_column(
         pl.col("a")
@@ -1543,7 +1627,7 @@ def test_cast_timezone() -> None:
         .alias("b")
     ).to_dict(False) == {
         "a": [datetime(2022, 9, 25, 18, 0)],
-        "b": [datetime(2022, 9, 25, 14, 0)],
+        "b": [datetime(2022, 9, 25, 10, 0, tzinfo=ny)],
     }
 
 
@@ -1553,3 +1637,59 @@ def test_tz_aware_get_idx_5010() -> None:
     )
     a = pa.array([when]).cast(pa.timestamp("s", tz="Asia/Shanghai"))
     assert int(pl.from_arrow(a)[0].timestamp()) == when  # type: ignore[union-attr]
+
+
+def test_tz_datetime_duration_arithm_5221() -> None:
+    run_datetimes = [
+        datetime.fromisoformat("2022-01-01T00:00:00+00:00"),
+        datetime.fromisoformat("2022-01-02T00:00:00+00:00"),
+    ]
+    out = pl.DataFrame(
+        data={"run_datetime": run_datetimes},
+        columns=[("run_datetime", pl.Datetime(time_zone="UTC"))],
+    )
+    utc = zoneinfo.ZoneInfo("UTC")
+    assert out.to_dict(False) == {
+        "run_datetime": [
+            datetime(2022, 1, 1, 0, 0, tzinfo=utc),
+            datetime(2022, 1, 2, 0, 0, tzinfo=utc),
+        ]
+    }
+
+
+def test_auto_infer_time_zone() -> None:
+    dt = datetime(2022, 10, 17, 10, tzinfo=zoneinfo.ZoneInfo("Asia/Shanghai"))
+    s = pl.Series([dt])
+    assert s.dtype == pl.Datetime("us", "Asia/Shanghai")
+    assert s[0] == dt
+
+
+def test_timezone_aware_date_range() -> None:
+    low = datetime(2022, 10, 17, 10, tzinfo=zoneinfo.ZoneInfo("Asia/Shanghai"))
+    high = datetime(2022, 11, 17, 10, tzinfo=zoneinfo.ZoneInfo("Asia/Shanghai"))
+
+    assert pl.date_range(low, high, interval=timedelta(days=5)).to_list() == [
+        datetime(2022, 10, 17, 10, 0, tzinfo=zoneinfo.ZoneInfo(key="Asia/Shanghai")),
+        datetime(2022, 10, 22, 10, 0, tzinfo=zoneinfo.ZoneInfo(key="Asia/Shanghai")),
+        datetime(2022, 10, 27, 10, 0, tzinfo=zoneinfo.ZoneInfo(key="Asia/Shanghai")),
+        datetime(2022, 11, 1, 10, 0, tzinfo=zoneinfo.ZoneInfo(key="Asia/Shanghai")),
+        datetime(2022, 11, 6, 10, 0, tzinfo=zoneinfo.ZoneInfo(key="Asia/Shanghai")),
+        datetime(2022, 11, 11, 10, 0, tzinfo=zoneinfo.ZoneInfo(key="Asia/Shanghai")),
+        datetime(2022, 11, 16, 10, 0, tzinfo=zoneinfo.ZoneInfo(key="Asia/Shanghai")),
+    ]
+
+    with pytest.raises(
+        ValueError,
+        match="Cannot mix different timezone aware datetimes. "
+        "Got: 'Asia/Shanghai' and 'None'",
+    ):
+        pl.date_range(
+            low, high.replace(tzinfo=None), interval=timedelta(days=5), time_zone="UTC"
+        )
+
+    with pytest.raises(
+        ValueError,
+        match="Given time_zone is different from that timezone aware datetimes. "
+        "Given: 'UTC', got: 'Asia/Shanghai'.",
+    ):
+        pl.date_range(low, high, interval=timedelta(days=5), time_zone="UTC")
