@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import typing
 from datetime import datetime
 from typing import TYPE_CHECKING
 
@@ -231,9 +232,11 @@ def test_join() -> None:
 
     joined = df_left.join(df_right, left_on="a", right_on="a").sort("a")
     assert joined["b"].series_equal(pl.Series("b", [1, 3, 2, 2]))
+
     joined = df_left.join(df_right, left_on="a", right_on="a", how="left").sort("a")
     assert joined["c_right"].is_null().sum() == 1
     assert joined["b"].series_equal(pl.Series("b", [1, 3, 2, 2, 4]))
+
     joined = df_left.join(df_right, left_on="a", right_on="a", how="outer").sort("a")
     assert joined["c_right"].null_count() == 1
     assert joined["c"].null_count() == 1
@@ -257,11 +260,12 @@ def test_join() -> None:
 
     # just check if join on multiple columns runs
     df_a.join(df_b, left_on=["a", "b"], right_on=["foo", "bar"])
-
     eager_join = df_a.join(df_b, left_on="a", right_on="foo")
-
     lazy_join = df_a.lazy().join(df_b.lazy(), left_on="a", right_on="foo").collect()
+
+    cols = ["a", "b", "bar", "ham"]
     assert lazy_join.shape == eager_join.shape
+    assert lazy_join.sort(by=cols).frame_equal(eager_join.sort(by=cols))
 
 
 def test_joins_dispatch() -> None:
@@ -480,3 +484,58 @@ def test_sorted_flag_after_joins() -> None:
     assert not joined["a"].flags["SORTED_ASC"]
     joined = dfb.join(dfa, on="b", how="anti")
     assert not joined["a"].flags["SORTED_ASC"]
+
+
+@typing.no_type_check
+def test_jit_sort_joins() -> None:
+    n = 200
+    dfa = pd.DataFrame(
+        {
+            "a": np.random.randint(0, 100, n),
+            "b": np.arange(0, n),
+        }
+    )
+
+    n = 40
+    dfb = pd.DataFrame(
+        {
+            "a": np.random.randint(0, 100, n),
+            "b": np.arange(0, n),
+        }
+    )
+    dfa_pl = pl.from_pandas(dfa).sort("a")
+    dfb_pl = pl.from_pandas(dfb)
+
+    for how in ["left", "inner"]:
+        pd_result = dfa.merge(dfb, on="a", how=how)
+        pd_result.columns = ["a", "b", "b_right"]
+
+        # left key sorted right is not
+        pl_result = dfa_pl.join(dfb_pl, on="a", how=how).sort(["a", "b"])
+
+        a = pl.from_pandas(pd_result).with_column(pl.all().cast(int)).sort(["a", "b"])
+        assert a.frame_equal(pl_result, null_equal=True)
+        assert pl_result["a"].flags["SORTED_ASC"]
+
+        # left key sorted right is not
+        pd_result = dfb.merge(dfa, on="a", how=how)
+        pd_result.columns = ["a", "b", "b_right"]
+        pl_result = dfb_pl.join(dfa_pl, on="a", how=how).sort(["a", "b"])
+
+        a = pl.from_pandas(pd_result).with_column(pl.all().cast(int)).sort(["a", "b"])
+        assert a.frame_equal(pl_result, null_equal=True)
+        assert pl_result["a"].flags["SORTED_ASC"]
+
+
+def test_asof_join_schema_5211() -> None:
+    df1 = pl.DataFrame({"today": [1, 2]})
+
+    df2 = pl.DataFrame({"next_friday": [1, 2]})
+
+    assert (
+        df1.lazy()
+        .join_asof(
+            df2.lazy(), left_on="today", right_on="next_friday", strategy="forward"
+        )
+        .schema
+    ) == {"today": pl.Int64, "next_friday": pl.Int64}
