@@ -87,7 +87,7 @@ def test_with_context() -> None:
         (df_a.with_context(df_b.lazy()).select(["a", "c"])).collect()
 
 
-def test_from_dicst_nested_nulls() -> None:
+def test_from_dicts_nested_nulls() -> None:
     assert pl.from_dicts([{"a": [None, None]}, {"a": [1, 2]}]).to_dict(False) == {
         "a": [[None, None], [1, 2]]
     }
